% This program computes estimates of the number of factors according
% to Hallin-Liska, Bai-Ng and our criterion

% Load eigenvalue data
load GUE30K.dat -ascii


% Declare data size, and other parameters
nn=150;
TT=500;
m=65;
rmax=10;
%RO=[0.4;0.6;0.8];
ro=0.2;
mpar1=2;
mpar2=1;
mpar3=.5;
qmax=4;
kmax=4;
r=2;
Gs(:,1)=((GUE30K(1,:)-GUE30K(2,:))./(GUE30K(2,:)-GUE30K(3,:)))';
for i=2:kmax
    Gs(:,i)=(max((GUE30K(1:i,:)-GUE30K(2:i+1,:))./(GUE30K(2:i+1,:)-GUE30K(3:i+2,:))))';
end
VARI=[1;8;16];
for aaa=1:3
%    ro=RO(aaa,1);
    vari=VARI(aaa,1);
for zai=1:500
    [aaa zai]

% Generate the AR and MA data

[data1,data2]=simulation_figure2_HLBN(nn,TT,r,vari,0,ro);

% Use first AR data and then MA data
for zz=1:2
    if zz==1
        data_big=data1;
    else
        data_big=data2;
    end
    
        for jj=3:-1:0
        n=nn-jj*10;
        T=TT-jj*10;
        data=data_big(1:n,1:T);
        MT=floor(0.7*sqrt(T));
        c=0.01:0.01:3;
        [cn,cm]=size(c);
        p1=(MT^(-2)+sqrt(MT)/sqrt(T)+1/n)*log(min([n;MT^2;sqrt(T)/sqrt(MT)]));
        p2=1/sqrt(min([n;MT^2;sqrt(T)/sqrt(MT)]));
        p3=log(min([n;MT^2;sqrt(T)/sqrt(MT)]))/(min([n;MT^2;sqrt(T)/sqrt(MT)]));
        IC1=zeros(qmax+1,1);
    
        % Do frequency domain calculations for HL criterion
        for l=-MT:MT
            %[jj l]
            theta=pi*l/(MT+0.5);
            SIGMA=data*data'/(2*pi*T);
            SIGMA1=zeros(n,n);
            for u=1:MT
                SIGMA1=SIGMA1+(1-abs(u/MT))*exp(-sqrt(-1)*u*theta)*data(:,u+1:end)*data(:,1:end-u)'/(2*pi*T);
            end
            SIGMA=SIGMA+SIGMA1+SIGMA1';
            lambda=real(eig(SIGMA));
            lambda=flipdim(sort(lambda),1);
            for k=0:qmax
                IC1(k+1,1)=IC1(k+1,1)+sum(lambda(k+1:end,1))/(n*(2*MT+1));
            end
        end

    
    
    % Use increasing data dimensions
    if jj==0
            % use Bai and Ng with rmax=10 to determine the number of static factors
    [V,D]=eig(data_big*data_big'/TT);
    [d,nd]=sort(diag(D));
    d=flipdim(d,1);
    nd=flipdim(nd,1);
    V=V(:,nd);
    Vhat=V(:,1:rmax);%first kmax eigenvectors
    dhat=d(1:rmax,1);%first kmax eigenvalues
    LAMBDA=Vhat*sqrt(nn);
    FACTORS=LAMBDA'*data_big/nn;
    ER=data_big-LAMBDA*FACTORS;
    sum1=sum(ER.^2);
    sighat=sum(sum1')/(nn*TT);
    IC2=0;

    for k=1:(rmax+1)
        if k==1
            sum1=sum((data_big).^2);
        else
            sum1=sum((data_big-LAMBDA(:,1:(k-1))*FACTORS(1:(k-1),:)).^2);
        end
        sum2=sum(sum1')/(nn*TT);
        IC2(k,1)=log(sum2)+(k-1)*((nn+TT)/(nn*TT))*log(min(nn,TT));
    end
    [waste,IC2nt]=min(IC2);
    ic2=IC2nt-1;
    if ic2==0
        ic2=1;
    else
    end
    % next get residuals from VAR(4) for factors and estimate 
    % their covariance and correlation matrices
    F=FACTORS(1:ic2,:);
    Y=F(:,5:end)';
    X=[F(:,1:end-4)' F(:,2:end-3)' F(:,3:end-2)' F(:,4:end-1)'];
    beta=inv(X'*X)*X'*Y;
    u=Y-X*beta;
    sigmau=u'*u/(TT-4);
    stanu=std(u);
    standard_u=u./kron(ones(TT-4,1),stanu);
    corru=standard_u'*standard_u/(TT-4);
    % Form statistics D1 and D2 and estimate the number of dynamic factors
    [v,d]=eig(sigmau);
    dcov=flipdim(sort(diag(d)),1);
    [v,d]=eig(corru);
    dcor=flipdim(sort(diag(d)),1);
    D1cov=sqrt(dcov.^2/(dcov'*dcov));
    D1cor=sqrt(dcor.^2/(dcor'*dcor));
    D2cov=sqrt(dcov'*dcov-(cumsum(dcov.^2))/(dcov'*dcov));
    D2cor=sqrt(dcor'*dcor-(cumsum(dcor.^2))/(dcor'*dcor));
    [ww,ww1]=size(D1cov);
    Q(zz,zai)=ww-sum(D1cov<mpar1/min(nn^(2/5),TT^(2/5)));
    Q(2+zz,zai)=ww-sum(D1cov<mpar2/min(nn^(2/5),TT^(2/5)));
    Q(4+zz,zai)=ww-sum(D1cov<mpar3/min(nn^(2/5),TT^(2/5)));
    Q(6+zz,zai)=ww-sum(D2cov<mpar1/min(nn^(2/5),TT^(2/5)));
    Q(8+zz,zai)=ww-sum(D2cov<mpar2/min(nn^(2/5),TT^(2/5)));
    Q(10+zz,zai)=ww-sum(D2cov<mpar3/min(nn^(2/5),TT^(2/5)));
    
    Q(12+zz,zai)=ww-sum(D1cor<mpar1/min(nn^(2/5),TT^(2/5)));
    Q(14+zz,zai)=ww-sum(D1cor<mpar2/min(nn^(2/5),TT^(2/5)));
    Q(16+zz,zai)=ww-sum(D1cor<mpar3/min(nn^(2/5),TT^(2/5)));
    Q(18+zz,zai)=ww-sum(D2cor<mpar1/min(nn^(2/5),TT^(2/5)));
    Q(20+zz,zai)=ww-sum(D2cor<mpar2/min(nn^(2/5),TT^(2/5)));
    Q(22+zz,zai)=ww-sum(D2cor<mpar3/min(nn^(2/5),TT^(2/5)));
    
        
        % Do frequency domain calculations for our criterion
        m=65;
        theta=linspace(0,pi-2*pi*m/T,MT);
        for l=1:MT
            %l
            freqdata=zeros(n,m);
            for jjj=1:m
                freqdata(:,jjj)=data*(exp(-sqrt(-1)*(1:T)*(theta(1,l)+2*pi*jjj/T)))';
            end
            %freqdata(1:5,1:5)
            %pause
            opts.disp=0;
            ddd(:,l)=eigs(freqdata'*freqdata,kmax+3,'lr',opts);
            s(:,l)=(ddd(1:kmax+1,l)-ddd(2:kmax+2,l))./(ddd(2:kmax+2,l)-ddd(3:kmax+3,l));
        end
        nfactors(jj+1,1)=test_criterion2a(real(s),real(ddd),Gs);
    end
            for k=0:qmax
            for ii=1:cm
                IC11(k+1,ii)=IC1(k+1,1)+k*c(1,ii)*p1;
                IC12(k+1,ii)=IC1(k+1,1)+k*c(1,ii)*p2;
                IC13(k+1,ii)=IC1(k+1,1)+k*c(1,ii)*p3;
                IC21(k+1,ii)=log(IC1(k+1,1))+k*c(1,ii)*p1;
                IC22(k+1,ii)=log(IC1(k+1,1))+k*c(1,ii)*p2;
                IC23(k+1,ii)=log(IC1(k+1,1))+k*c(1,ii)*p3;
            end
        end
        [waste,q11]=min(IC11);
        Q11(jj+1,:)=q11-1;
        [waste,q12]=min(IC12);
        Q12(jj+1,:)=q12-1;
        [waste,q13]=min(IC13);
        Q13(jj+1,:)=q13-1;
        [waste,q21]=min(IC21);
        Q21(jj+1,:)=q21-1;
        [waste,q22]=min(IC22);
        Q22(jj+1,:)=q22-1;
        [waste,q23]=min(IC23);
        Q23(jj+1,:)=q23-1;

    end % end of jj cycle
    
        S2(1,:)=sum((Q11-ones(4,1)*sum(Q11)/4).^2)/4;
    S2(2,:)=sum((Q12-ones(4,1)*sum(Q12)/4).^2)/4;
    S2(3,:)=sum((Q13-ones(4,1)*sum(Q13)/4).^2)/4;
    S2(4,:)=sum((Q21-ones(4,1)*sum(Q21)/4).^2)/4;
    S2(5,:)=sum((Q22-ones(4,1)*sum(Q22)/4).^2)/4;
    S2(6,:)=sum((Q23-ones(4,1)*sum(Q23)/4).^2)/4;
    
    % find the second stability region used by HL criterion
    for bbb=1:6
        iii=1;
        ff=S2(bbb,iii);
        while ff==0
            if iii==cm
                ff=1;
            else
                iii=iii+1;
                ff=S2(bbb,iii);
            end
        end
        while ff~=0
            if iii==cm
                ff=0;
            else
                iii=iii+1;
                ff=S2(bbb,iii);
            end
        end
        if bbb==1
            HL((bbb-1)*2+zz,zai)=Q11(1,iii);
        elseif bbb==2
            HL((bbb-1)*2+zz,zai)=Q12(1,iii);
        elseif bbb==3
            HL((bbb-1)*2+zz,zai)=Q13(1,iii);
        elseif bbb==4
            HL((bbb-1)*2+zz,zai)=Q21(1,iii);
        elseif bbb==5
            HL((bbb-1)*2+zz,zai)=Q22(1,iii);
        else
            HL((bbb-1)*2+zz,zai)=Q23(1,iii);
        end
    end

    
    % Complete calculations for my criterion
    for j=1:kmax
        NFACTORS(j,1)=sum(nfactors==j);
    end
    [aa,bb]=max(NFACTORS);
    ONA(zz,zai)=bb;
end% end of zz cycle
end% end of zai cycle
if vari==1
    save HL_1.dat HL -ascii
    save ONA_1.dat ONA -ascii
    save BN_1.dat Q -ascii
elseif vari==8
    save HL_8.dat HL -ascii
    save ONA_8.dat ONA -ascii
    save BN_8.dat Q -ascii
else
    save HL_16.dat HL -ascii
    save ONA_16.dat ONA -ascii
    save BN_16.dat Q -ascii
end
end% end of aaa cycle
